package io.tswf.transmutations.common.remapping

import groovy.transform.CompileStatic
import groovy.transform.Internal
import org.objectweb.asm.Type
import org.objectweb.asm.commons.Method

import java.util.function.Function

@CompileStatic
class TransmutationRemapper {

    @Lazy
    static List<ITransmutationRemappingStrategy> remappingStrategies = ServiceLoader.load(ITransmutationRemappingStrategy)
            .toList()
            .sortBy(true) { it.priority }

    static Type mapType(Type anotherType) {
        return _delegateToStrategies {it.mapType(anotherType)  } ?: anotherType
    }

    static Method mapMethod(Type anotherMethodOwner, Method anotherMethod) {
        return _delegateToStrategies {it.mapMethod(anotherMethodOwner, anotherMethod)  } ?: anotherMethod
    }

    static Tuple2<String, Type> mapField(Type anotherFieldOwner, String anotherFieldName, Type anotherFieldType) {
        return _delegateToStrategies { it.mapField(anotherFieldOwner, anotherFieldName, anotherFieldType) } ?: new Tuple2<>(anotherFieldName, anotherFieldType)
    }

    @Internal
    static <T> T _delegateToStrategies(Function<ITransmutationRemappingStrategy, T> delegate) {
        def result = (Type) null

        for (def strategy : remappingStrategies) {
            result = delegate.apply(strategy)
            if (result != null) {
                return result
            }
        }

        return null
    }
}
